#include "StdAfx.h"
#include "enemy.h"
#include "game.h"
#include <fstream>

using namespace std;

const int train_w = 40, train_h = 7;

enemy::enemy(std::string objectfile)
{
	fill();
	load_objectfile(objectfile);
}

enemy::enemy(const enemy &from_blueprint, int posx, int posy, int _facing)
{
	fill();
	pic = from_blueprint.get_pic();
	destroy_graphics = false;
	pos[0] = posx;
	pos[1] = posy;
	facing = _facing;
	health = from_blueprint.get_health();
	speed = from_blueprint.get_speed();
	shoot_speed = from_blueprint.shoot_speed;
	weapon = from_blueprint.get_weapon();
	ai = from_blueprint.get_ai();
}

enemy::~enemy()
{
	if(destroy_graphics == true && pic)
	{
		delete pic;
		pic = 0;
	}
}

void enemy::fill()
{
	object::fill();
	facing = 2;
	health = 100;
	speed = 1;
	shoot_speed = 1;
	weapon = 0;
	ai = 1;
	shoot_haswaited = 0;
	walk_haswaited = 0;
}


void enemy::write_objectfile(std::string to, std::string picpath, int health, int speed, int shoot_speed, int weapon, int ai)
{
	ofstream file(to.c_str(), ios::out | ios::trunc | ios::binary);
	if(!file)
	{
		error::fatalError("Unable to write a file.", false);
		file.close();
		return;	
	}
	
	file.write((char*)&health, sizeof(int));
	file.write((char*)&speed, sizeof(int));
	file.write((char*)&shoot_speed, sizeof(int));
	file.write((char*)&weapon, sizeof(int));
	file.write((char*)&ai, sizeof(int));
	file << picpath;
	file.close();	
}

void enemy::load_objectfile(std::string from)
{
	ifstream file(from.c_str(), ios::in | ios::binary);
	if(!file)
	{
		error::fatalError("Unable to open a file.", true);
		file.close();
		return;	
	}
	
	string picpath;
	getline(file, picpath, '\n');
	while(picpath[picpath.length() - 1] == '\n' || picpath[picpath.length() - 1] == '\r')
		picpath.erase(picpath.length() - 1);
	file >> health >> speed >> shoot_speed >> weapon >> ai;
	pic = new asciibmp(picpath);
	destroy_graphics = true;
	
	file.close();	
}

bullet* enemy::do_ai(int trainx)
{
	walk_haswaited += 2;	
	shoot_haswaited += 2;
	
	bool has_shot = false;
	int shoot_dir = 0;
	const int trainy = 33;
	
	if(ai >= 2 && speed) //walk
	{
		while(walk_haswaited >= speed)
		{
			walk_haswaited -= speed;
			
			/* TODO (Deathlord#1#): Insert collision detection with train. train dereived from 
			                        object. */
			
			pos[0] += move_pattern[facing][0];
			pos[1] += move_pattern[facing][1];
		}
	}
	if(ai >= 1) //shoot
	{
		if(shoot_haswaited >= shoot_speed)
		{
			shoot_haswaited -= shoot_speed;
			has_shot = true;
			
			if(pos[1] + pic->getH() / 2 >= trainy && pos[1] + pic->getH() <= trainy + train_h) //next to train in y-dir
			{
				if(pos[0] <= trainx + train_w) //left of trainmiddle
				{
					shoot_dir = 2;
				}
				else //right of trainmiddle
				{
					shoot_dir = 6;
				}	
			}
			else
			{
				if(pos[1] + pic->getH() < trainy) //north of train in y-dir
				{
					if(pos[0] + pic->getW() / 2 < trainx) //left of train
					{
						shoot_dir = 3;
					}
					else if(pos[0] > trainx + train_w) //right of train
					{
						shoot_dir = 5;
					}
					else //above train
					{
						shoot_dir = 4;
					}
				}
				else //south of train in y-dir
				{
					if(pos[0] + pic->getW() / 2 < trainx) //left of train
					{
						shoot_dir = 1;
					}
					else if(pos[0] > trainx + train_w) //right of train
					{
						shoot_dir = 7;
					}
					else //above train
					{
						shoot_dir = 0;
					}
				} 
			}
		}
	}
	
	/* DONE (Deathlord#1#): Returns pointer to bullet if enemy shoots */
	if(has_shot == true)
	{
	
		bullet *ret_b = new bullet;
		ret_b->set_facing(shoot_dir);
		ret_b->set_owner(1);
	
		if(weapon == 0) //mg
		{
			ret_b->set_speed(1);
			ret_b->set_damage(5);
			ret_b->set_range(100);
			mg.play(0, 180);
		}
		if(weapon == 1) //rocket launcher
		{
			ret_b->set_speed(2);
			ret_b->set_damage(13);
			ret_b->set_range(150);
			rocket.play(0, 180);
		}
		
		ret_b->set_pos(pos[0], pos[1]);
		
		return ret_b;
	}
	return NULL;
}
	
